from . import tools, utils


class ConjureTraversalList:
    def __init__(
            self,
            custom_payload: list = None,
            web_files: list = None,
            default_payload: bool = True,
            url_encoded: bool = True,
            null_bytes: bool = True,
            repeat: int = 5
    ):

        self.custom_payload = custom_payload
        self.default_payload = default_payload
        self.url_encoded = url_encoded
        self.null_bytes = null_bytes
        self.repeat = repeat + 1

        self.web_files = list()
        if web_files:
            self.web_files = self.web_files + web_files

        self.traversal_list = list()
        self.traversal_list_length = 0

    def _update(self, traversal_list: list):
        self.traversal_list = self.traversal_list + traversal_list
        self.traversal_list_length = len(self.traversal_list)

    def create(self) -> list:
        directives = [f"{directive * i}" for i in range(2, self.repeat) for directive in tools.directives]
        self._update(traversal_list=directives)

        if self.default_payload:
            default_payload_included = utils.add_payload(traversal_list=directives, payloads=tools.payloads)
            self._update(traversal_list=default_payload_included)

        if self.custom_payload:
            custom_payload_included = utils.add_payload(traversal_list=directives, payloads=self.custom_payload)
            self._update(traversal_list=custom_payload_included)

        if self.web_files:
            web_files_included = utils.add_web_file(traversal_list=self.traversal_list, web_files=self.web_files)
            self._update(traversal_list=web_files_included)

        if self.url_encoded:
            url_encoded_included = utils.full_url_encoder(unencoded=self.traversal_list)
            self._update(traversal_list=url_encoded_included)

        if self.null_bytes:
            null_bytes_included = utils.add_null_bytes(traversal_list=self.traversal_list, null_bytes=tools.null_bytes)
            self._update(traversal_list=null_bytes_included)

        return self.traversal_list



import os

"""
File System Magic
"""


def find(name, path, file_type: str = None):
    if file_type not in ['file', 'dir', None]:
        raise ValueError(f"input 'file_type' should be of value 'file', 'dir' or 'None'.")

    for root, dirs, files in os.walk(path):
        if file_type == 'file':
            if name in files:
                return os.path.join(root, name)
        elif file_type == 'dir':
            if name in dirs:
                return os.path.join(root, name)
        else:
            if name in dirs or name in files:
                return os.path.join(root, name)


def find_all(name, path, file_type: str = None):
    result = []
    if file_type not in ['file', 'dir', None]:
        raise ValueError(f"input 'file_type' should be of value 'file', 'dir' or 'None'.")

    for root, dirs, files in os.walk(path):
        if file_type == 'file':
            if name in files:
                result.append(os.path.join(root, name))
        elif file_type == 'dir':
            if name in dirs:
                result.append(os.path.join(root, name))
        else:
            if name in dirs or name in files:
                result.append(os.path.join(root, name))
    return result


def read_wordlist(file: str, wordlist: list = None):
    """
    Quick way to load a new wordlist from a file.
    :param file: Wordlist location
    :param wordlist: Append file contents to existing wordlist.
    :return: List of words
    """

    if not wordlist:
        wordlist = []

    if file and not os.path.isfile(file):
        raise FileNotFoundError(f"Cannot locate file {file}. Please ensure it exists.")

    with open(file, 'r') as f:
        for file_line in f:
            # skip line if starts with a comment
            if file_line.startswith("#"):
                continue
            wordlist.append(file_line.strip())
    return wordlist



HEADER = "[SC] "

CONNECTION_CLOSED = f"{HEADER}Connection closed."
NO_CLIENT = f"{HEADER}No client connections."
NO_DATA = f"{HEADER}No data was provided to send."
SENT = f"{HEADER}Sent."
SERVER_STARTED = f"{HEADER}Server started."
TIMEOUT_RECEIVED = f"{HEADER}Send / Receive timed out."




import urllib.parse


def add_null_bytes(traversal_list: list, null_bytes: list):
    return [f"{direct}{null_byte}" for direct in traversal_list for null_byte in null_bytes]


def add_payload(traversal_list: list, payloads: list):
    return [f"{direct}{payload}" for direct in traversal_list for payload in payloads]


def add_web_file(traversal_list: list, web_files: list):
    return [f"{file}/{direct}" for direct in traversal_list for file in web_files]


def full_url_encoder(unencoded: str | list):

    def encoder(plain_text: str) -> str:
        url_encode = urllib.parse.quote(plain_text)
        return url_encode.replace('.', '%2E')

    if isinstance(unencoded, str):
        return encoder(plain_text=unencoded)

    return [encoder(plain_text=traverse) for traverse in unencoded]
